from pathlib import Path


def appendID(filename: str, id: str) -> str:
    # https://stackoverflow.com/a/37487898
    p = Path(filename)
    return "{0}_{2}{1}".format(Path.joinpath(p.parent, p.stem), p.suffix, id)



import itertools
import json
import os
from argparse import ArgumentParser
from datetime import datetime
from functools import reduce
from os.path import exists, join

from dateutil.parser import parse as dateParse
from progress.bar import Bar


# Command line arguement parsing
def get_argparse() -> ArgumentParser:
    parser: ArgumentParser = ArgumentParser(
        prog="ssl-metrics-git-commits-loc",
        usage="This program outputs the lines of code (LOC) per commit and the delta LOC of a Git repository in JSON format.",
    )
    parser.add_argument(
        "-d",
        "--directory",
        help="Directory containing repository root folder (.git)",
        default=".",
        type=str,
        required=True,
    )
    parser.add_argument(
        "-b",
        "--branch",
        help="Git branch for analysis to be ran on",
        default="main",
        type=str,
        required=True,
    )
    parser.add_argument(
        "-o",
        "--output",
        help="Output analysis to a JSON file",
        type=str,
        required=True,
    )
    parser.add_argument(
        "-c",
        "--cores",
        help="Number of cores to use for analysis",
        type=int,
        required=False,
        default=1,
    )
    return parser


def repoExists(directory: str = ".") -> bool:
    if exists(join(directory, ".git")) is False:
        return False
    return True


def parseCommitLineFromLog(line: str) -> dict:
    splitLine: list = line.split(";")
    name: str = splitLine[0]
    email: str = splitLine[1]
    hash: str = splitLine[2]
    date = splitLine[-1]
    return {
        "author_name": name,
        "author_email": email,
        "hash": hash,
        "date": dateParse(date),
    }


def analyzeCommits(commits: list, date0: datetime):
    loc_sum: int = 0
    commitCounter: int = 0
    with Bar("Processing commits... ", max=len(commits) - 1) as ib:
        for index in range(len(commits) - 1):
            authorName: str = commits[index]["author_name"]
            authorEmail: str = commits[index]["author_email"]
            hashX: str = commits[index]["hash"]
            hashY: str = commits[index + 1]["hash"]
            commitDate: str = commits[index]["date"].strftime("%m/%d/%Y")
            dateY: datetime = commits[index + 1]["date"]

            gdf: dict = gitDiffTree(hashX, hashY)

            loc = map(lambda info: info["loc"], gdf)

            delta_sum = reduce(lambda x, y: x + y, loc, 0)

            loc_sum += delta_sum
            commit_day = (dateY - date0).days
            result = {
                "commit_number": commitCounter,
                "author_name": authorName,
                "author_email": authorEmail,
                "hash": hashY,
                "delta_loc": delta_sum,
                "loc_sum": loc_sum,
                "kloc": float(loc_sum / 1000),
                "commit_date": commitDate,
                "days_since_0": commit_day,
            }

            commitCounter += 1
            ib.next()
            yield result


def gitDiffTree(hashX: str, hashY: str) -> dict:
    # git diff help page: https://www.git-scm.com/docs/git-diff
    with os.popen(f"git diff-tree -r {hashX} {hashY}") as diffTreePipe:
        for line in diffTreePipe:

            lineInfo: dict = parseDiffTreeLine(line)

            lineStatus: str = lineInfo.get("status")
            if lineStatus == "A":
                addLines(lineInfo)
            elif lineStatus == "D":
                deleteLines(lineInfo)
            elif lineStatus == "M":
                deltaLines(lineInfo)
            else:
                continue
            yield lineInfo


def parseDiffTreeLine(line: str) -> dict:
    tokens: list = line.split()
    try:
        (dstMode, sha1Src, sha1Dst, status, srcPath) = tokens[1:6]
    except ValueError:
        return {}
    return {
        "dstMode": dstMode,
        "sha1Src": sha1Src,
        "sha1Dst": sha1Dst,
        "status": status,
        "srcPath": srcPath,
    }


def addLines(line: dict) -> None:
    line["loc"] = countFileLines(line["sha1Dst"])


def deleteLines(line: dict) -> None:
    line["loc"] = -countFileLines(line["sha1Src"])


def deltaLines(line: dict) -> None:
    loc_before = countFileLines(line["sha1Src"])
    loc_after = countFileLines(line["sha1Dst"])
    line["loc"] = loc_after - loc_before


# TODO: Could replace this with cloc; make parametric
def countFileLines(blobHash: str):
    # Git show help page: https://www.git-scm.com/docs/git-show
    # wc help page: https://linux.die.net/man/1/wc
    with os.popen(f"git show {blobHash} | wc -l") as data:
        return int(data.read().split()[0])
    return -1


def exportJSON(filename, commitInfo):
    with open(filename, "w") as jsonf:
        json.dump(commitInfo, jsonf, indent=4)


def pairwise(
    iterable,
    coreCount: int,
    maxValue: int,
) -> list:
    data: list = []
    a, b = itertools.tee(iterable)
    next(b, None)

    x: tuple
    for x in zip(a, b):
        temp: list = [x[0], x[1]]
        if temp[0] == 0:
            pass
        else:
            temp[0] += 1
        data.append(temp)

    if len(data) < coreCount:
        data.append([data[-1][1] + 1, maxValue])
        print(data)

    if data[-1][1] != maxValue:
        data[-1][1] = maxValue

    return data


def main() -> bool:
    pwd = os.getcwd()
    args = get_argparse().parse_args()

    if repoExists(directory=args.directory) is False:
        return False

    os.chdir(args.directory)
    os.system(f"git checkout {args.branch}")

    # Git log help page: https://www.git-scm.com/docs/git-log
    with os.popen(r'git log --reverse --pretty=format:"%an;%ae;%H;%ci"') as gitLogPipe:
        commits: list = [parseCommitLineFromLog(line=commit) for commit in gitLogPipe]

        commit0AuthorName: str = commits[0]["author_name"]
        commit0AuthorEmail: str = commits[0]["author_email"]
        commit0Date: datetime = commits[0]["date"]
        commits = [
            {
                "author_name": commit0AuthorName,
                "author_email": commit0AuthorEmail,
                "hash": "--root",
                "date": commit0Date,
            }
        ] + commits

        commit_info_iter = analyzeCommits(commits=commits, date0=commits[0]["date"])
        commit_info = list(commit_info_iter)
        delta_loc_iter = map(lambda info: info["delta_loc"], commit_info)
        loc_sum = reduce(lambda x, y: x + y, delta_loc_iter, 0)

    if args.output:
        exportJSON(join(pwd, args.output), commit_info)

    os.chdir(pwd)
    return True


if __name__ == "__main__":
    main()



from textwrap import wrap

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.figure import Figure

from ssl_metrics_git_commits_loc.polynomialMath import findBestFitLine


def graph(
    title: str,
    xLabel: str,
    yLabel: str,
    xData: list,
    yData: list,
    maximumDegree: int = None,
    bestFit: bool = False,
    velocity: bool = False,
    acceleration: bool = False,
) -> Figure:
    figure: Figure = plt.figure()
    plt.title("\n".join(wrap(title, width=60)))
    plt.xlabel(xlabel=xLabel)
    plt.ylabel(ylabel=yLabel)

    if type(maximumDegree) is int:
        data: tuple = findBestFitLine(
            x=xData,
            y=yData,
            maximumDegree=maximumDegree,
        )
        bfModel: np.poly1d = data[1]
        line: np.ndarray = np.linspace(0, max(xData), 100)

    if bestFit:
        plt.plot(line, bfModel(line))

    elif velocity:
        velocityModel = np.polyder(p=bfModel, m=1)
        plt.plot(line, velocityModel(line))

    elif acceleration:
        accelerationModel = np.polyder(p=bfModel, m=2)
        plt.plot(line, accelerationModel(line))

    else:
        plt.plot(xData, yData)

    plt.tight_layout()
    return figure


def graphAll(
    title: str,
    xLabel: str,
    yLabelList: list,
    xData: list,
    yData: list,
    maximumDegree: int,
    subplotTitles: list,
) -> Figure:
    figure: Figure = plt.figure()
    plt.suptitle(title)

    # Data
    plt.subplot(2, 2, 1)
    plt.xlabel(xlabel=xLabel)
    plt.ylabel(ylabel=yLabelList[0])
    plt.title(subplotTitles[0])
    plt.plot(xData, yData)
    plt.tight_layout()

    # Best Fit
    plt.subplot(2, 2, 2)
    data: tuple = findBestFitLine(x=xData, y=yData, maximumDegree=maximumDegree)
    bfModel: np.poly1d = data[1]
    line: np.ndarray = np.linspace(0, max(xData), 100)
    plt.ylabel(ylabel=yLabelList[1])
    plt.xlabel(xlabel=xLabel)
    plt.title(subplotTitles[1])
    plt.plot(line, bfModel(line))
    plt.tight_layout()

    # Velocity
    plt.subplot(2, 2, 3)
    velocityModel = np.polyder(p=bfModel, m=1)
    line: np.ndarray = np.linspace(0, max(xData), 100)
    plt.ylabel(ylabel=yLabelList[2])
    plt.xlabel(xlabel=xLabel)
    plt.title(subplotTitles[2])
    plt.plot(line, velocityModel(line))
    plt.tight_layout()

    # Acceleration
    plt.subplot(2, 2, 4)
    accelerationModel = np.polyder(p=bfModel, m=2)
    line: np.ndarray = np.linspace(0, max(xData), 100)
    plt.ylabel(ylabel=yLabelList[3])
    plt.xlabel(xlabel=xLabel)
    plt.title(subplotTitles[3])
    plt.plot(line, accelerationModel(line))
    plt.tight_layout()

    return figure



from argparse import ArgumentParser, Namespace

import pandas
from matplotlib.figure import Figure
from pandas import DataFrame

from ssl_metrics_git_commits_loc.fileOperations import appendID
from ssl_metrics_git_commits_loc.graphing import graph, graphAll


def getArgparse() -> Namespace:
    parser: ArgumentParser = ArgumentParser(
        prog="ssl-metrics-git-commits-loc Graph Generator",
        usage="This is a proof of concept demonstrating that it is possible to use Git commits to compute metrics.",
        description="The default action is to graph all figures of LOC on a single chart. If multiple data and/or graphing options are choosen the output filename and the title of the figure/chartwill reflect the combination that is being graphed.",
    )
    parser.add_argument(
        "-i",
        "--input",
        help="The input data file that will be read to create the graphs",
        type=str,
        required=True,
    )
    parser.add_argument(
        "-o",
        "--output",
        help="The filename to output the graph to",
        type=str,
        required=True,
    )
    parser.add_argument(
        "-r",
        "--repository-name",
        help="Name of the repository that is being analyzed. Will be used in the graph title",
        type=str,
        required=True,
    )
    parser.add_argument(
        "--loc", help="Utilize LOC data", required=False, action="store_true"
    )
    parser.add_argument(
        "--dloc", help="Utilize Delta LOC data", required=False, action="store_true"
    )
    parser.add_argument(
        "--kloc", help="Utilize KLOC data", required=False, action="store_true"
    )
    parser.add_argument(
        "--graph-data",
        help="Graph the raw data. Discrete graph of the data",
        required=False,
        action="store_true",
    )
    parser.add_argument(
        "--graph-best-fit",
        help="Graph the best fit polynomial of the data. Continous graph of the data. Polynomial degrees can be configured with `-m`",
        required=False,
        action="store_true",
    )
    parser.add_argument(
        "--graph-velocity",
        help="Graph the velocity of the data. Computes the best fit polynomial and takes the first derivitve. Polynomial degrees can be configured with `-m`",
        required=False,
        action="store_true",
    )
    parser.add_argument(
        "--graph-acceleration",
        help="Graph the acceleration of the data. Computes the best fit polynomial and takes the second derivitve. Polynomial degrees can be configured with `-m`",
        required=False,
        action="store_true",
    )
    parser.add_argument(
        "--graph-all",
        help="Graphs all possible figures of the data onto one chart. Computes the best fit polynomial and takes the first and second derivitve. Polynomial degrees can be configured with `-m`",
        required=False,
        action="store_true",
    )
    parser.add_argument(
        "--x-min",
        help="The smallest x value that will be plotted",
        type=int,
        required=False,
        default=0,
    )
    parser.add_argument(
        "--x-max",
        help="The largest x value that will be plotted",
        type=int,
        required=False,
        default=-1,
    )
    parser.add_argument(
        "-m",
        "--maximum-polynomial-degree",
        help="Estimated maximum degree of the best fit polynomial",
        type=int,
        required=False,
        default=15,
    )
    parser.add_argument(
        "-s",
        "--stepper",
        help="Step through every nth data point",
        type=int,
        required=False,
        default=1,
    )
    return parser.parse_args()


def graphChart(
    figureType: str,
    title: str,
    xLabel: str,
    yLabel: str,
    xData: list,
    yData: list,
    filename: str,
    maximumDegree: int = None,
    subplotTitles: list = None,
    yLabelList: list = None,
) -> None:
    if figureType == "data":
        figure: Figure = graph(
            title=title,
            xLabel=xLabel,
            yLabel=yLabel,
            xData=xData,
            yData=yData,
        )
    if figureType == "best_fit":
        figure: Figure = graph(
            title=title,
            xLabel=xLabel,
            yLabel=yLabel,
            xData=xData,
            yData=yData,
            maximumDegree=maximumDegree,
            bestFit=True,
        )
    if figureType == "velocity":
        figure: Figure = graph(
            title=title,
            xLabel=xLabel,
            yLabel=yLabel,
            xData=xData,
            yData=yData,
            maximumDegree=maximumDegree,
            velocity=True,
        )
    if figureType == "acceleration":
        figure: Figure = graph(
            title=title,
            xLabel=xLabel,
            yLabel=yLabel,
            xData=xData,
            yData=yData,
            maximumDegree=maximumDegree,
            acceleration=True,
        )
    if figureType == "all":
        figure: Figure = graphAll(
            title=title,
            xLabel=xLabel,
            xData=xData,
            yData=yData,
            maximumDegree=maximumDegree,
            subplotTitles=subplotTitles,
            yLabelList=yLabelList,
        )

    figure.savefig(filename)
    figure.clf()


def main() -> None:
    def _graphDataChart(title: str, yLabel: str, yData: list, filename: str) -> None:
        graphChart(
            figureType="data",
            title=title,
            xLabel=xLabel,
            yLabel=yLabel,
            xData=xData,
            yData=yData,
            filename=filename,
        )

    def _graphBestFitChart(title: str, yLabel: str, yData: list, filename: str) -> None:
        graphChart(
            figureType="best_fit",
            title=title,
            xLabel=xLabel,
            yLabel=yLabel,
            xData=xData,
            yData=yData,
            filename=filename,
            maximumDegree=args.maximum_polynomial_degree,
        )

    def _graphVelocityChart(
        title: str, yLabel: str, yData: list, filename: str
    ) -> None:
        graphChart(
            figureType="velocity",
            title=title,
            xLabel=xLabel,
            yLabel=yLabel,
            xData=xData,
            yData=yData,
            filename=filename,
            maximumDegree=args.maximum_polynomial_degree,
        )

    def _graphAccelerationChart(
        title: str, yLabel: str, yData: list, filename: str
    ) -> None:
        graphChart(
            figureType="acceleration",
            title=title,
            xLabel=xLabel,
            yLabel=yLabel,
            xData=xData,
            yData=yData,
            filename=filename,
            maximumDegree=args.maximum_polynomial_degree,
        )

    def _graphAllCharts(
        title: str, yLabelList: list, yData: list, filename: str
    ) -> None:
        graphChart(
            figureType="all",
            title=title,
            xLabel=xLabel,
            yLabel=None,
            xData=xData,
            yData=yData,
            filename=filename,
            maximumDegree=args.maximum_polynomial_degree,
            subplotTitles=[
                "Data",
                "Best Fit",
                "Velocity",
                "Acceleration",
            ],
            yLabelList=yLabelList,
        )

    args: Namespace = getArgparse()

    if args.input[-5::] != ".json":
        print("Invalid input file type. Input file must be JSON")
        quit(1)
    if args.x_min < 0:
        print("Invalid x window min. X window min >= 0")
        quit(2)
    if args.maximum_polynomial_degree < 1:
        print(
            "The maximum degree polynomial is too small. Maximum degree polynomial >= 1"
        )
        quit(3)
    if args.stepper < 1:
        print("The stepper is too small. Stepper >= 1")
        quit(4)

    if (args.loc is False) and (args.dloc is False) and (args.kloc is False):
        print("No data option choosen. Defaulting to --loc")
        args.loc = True
    if (
        (args.graph_data is False)
        and (args.graph_best_fit is False)
        and (args.graph_velocity is False)
        and (args.graph_acceleration is False)
        and (args.graph_all is False)
    ):
        print("No graph option choosen. Defaulting to --graph-all")
        args.graph_all = True

    xLabel: str = f"Every {args.stepper} Commit(s)"
    yLabel0: str = "{}"
    yLabel1: str = "d/dx {}"
    yLabel2: str = "d^2/dx^2 {}"

    df: DataFrame = pandas.read_json(args.input)

    t: str = (
        lambda typeOfGraph, repositoryName, yUnits: f"{typeOfGraph}{repositoryName} {yUnits} / Every {args.stepper} Commits"
    )
    x: list = lambda maxValue: [x for x in range(len(df["loc_sum"]))][
        args.x_min : maxValue : args.stepper
    ]
    y: list = lambda column, maxValue: df[column].tolist()[
        args.x_min : maxValue : args.stepper
    ]

    if args.x_max <= -1:
        xData: list = x(-1)
        yLOC: list = y("loc_sum", -1)
        yDLOC: list = y("delta_loc", -1)
        yKLOC: list = y("kloc", -1)
    else:
        xData: list = x(args.x_max + 1)
        yLOC: list = y("loc_sum", args.x_max + 1)
        yDLOC: list = y("delta_loc", args.x_max + 1)
        yKLOC: list = y("kloc", args.x_max + 1)

    if args.loc:
        if args.graph_data:
            title: str = t("", args.repository_name, "LOC")
            filename: str = appendID(filename=args.output, id="loc_data")
            _graphDataChart(
                title=title,
                yLabel=yLabel0.format("LOC"),
                yData=yLOC,
                filename=filename,
            )

        if args.graph_best_fit:
            title: str = t("Best Fit of ", args.repository_name, "LOC")
            filename: str = appendID(filename=args.output, id="loc_best_fit")
            _graphBestFitChart(
                title=title,
                yLabel=yLabel0.format("LOC"),
                yData=yLOC,
                filename=filename,
            )

        if args.graph_velocity:
            title: str = t("Velocity of ", args.repository_name, "LOC")
            filename: str = appendID(filename=args.output, id="loc_velocity")
            _graphVelocityChart(
                title=title,
                yLabel=yLabel1.format("LOC"),
                yData=yLOC,
                filename=filename,
            )

        if args.graph_acceleration:
            title: str = t("Acceleration of ", args.repository_name, "LOC")
            filename: str = appendID(filename=args.output, id="loc_acceleration")
            _graphAccelerationChart(
                title=title,
                yLabel=yLabel2.format("LOC"),
                yData=yLOC,
                filename=filename,
            )

        if args.graph_all:
            filename: str = appendID(filename=args.output, id="loc_all")
            title = t("", args.repository_name, "LOC")
            yLabelList: list = [
                yLabel0.format("LOC"),
                yLabel0.format("LOC"),
                yLabel1.format("LOC"),
                yLabel2.format("LOC"),
            ]
            _graphAllCharts(
                title=title,
                yLabelList=yLabelList,
                yData=yLOC,
                filename=filename,
            )

    if args.dloc:
        if args.graph_data:
            title: str = t("", args.repository_name, "DLOC")
            filename: str = appendID(filename=args.output, id="dloc_data")
            _graphDataChart(
                title=title,
                yLabel=yLabel0.format("DLOC"),
                yData=yDLOC,
                filename=filename,
            )

        if args.graph_best_fit:
            title: str = t("Best Fit of ", args.repository_name, "DLOC")
            filename: str = appendID(filename=args.output, id="dloc_best_fit")
            _graphBestFitChart(
                title=title,
                yLabel=yLabel0.format("DLOC"),
                yData=yDLOC,
                filename=filename,
            )

        if args.graph_velocity:
            title: str = t("Velocity of ", args.repository_name, "DLOC")
            filename: str = appendID(filename=args.output, id="dloc_velocity")
            _graphVelocityChart(
                title=title,
                yLabel=yLabel1.format("DLOC"),
                yData=yDLOC,
                filename=filename,
            )

        if args.graph_acceleration:
            title: str = t("Acceleration of ", args.repository_name, "DLOC")
            filename: str = appendID(filename=args.output, id="dloc_acceleration")
            _graphAccelerationChart(
                title=title,
                yLabel=yLabel2.format("DLOC"),
                yData=yDLOC,
                filename=filename,
            )

        if args.graph_all:
            title: str = t("", args.repository_name, "DLOC")
            filename: str = appendID(filename=args.output, id="dloc_all")
            yLabelList: list = [
                yLabel0.format("DLOC"),
                yLabel0.format("DLOC"),
                yLabel1.format("DLOC"),
                yLabel2.format("DLOC"),
            ]
            _graphAllCharts(
                title=title,
                yLabelList=yLabelList,
                yData=yDLOC,
                filename=filename,
            )

    if args.kloc:
        if args.graph_data:
            title: str = t("", args.repository_name, "KLOC")
            filename: str = appendID(filename=args.output, id="kloc_data")
            _graphDataChart(
                title=title,
                yLabel=yLabel0.format("KLOC"),
                filename=filename,
            )

        if args.graph_best_fit:
            title: str = t(
                "Best Fit of ",
                args.repository_name,
                "KLOC",
            )
            filename: str = appendID(filename=args.output, id="kloc_best_fit")
            _graphBestFitChart(
                title=title,
                yLabel=yLabel0.format("KLOC"),
                yData=yKLOC,
                filename=filename,
            )

        if args.graph_velocity:
            title: str = t(
                "Velocity of ",
                args.repository_name,
                "KLOC",
            )
            filename: str = appendID(filename=args.output, id="kloc_velocity")
            _graphVelocityChart(
                title=title,
                yLabel=yLabel1.format("KLOC"),
                yData=yKLOC,
                filename=filename,
            )

        if args.graph_acceleration:
            title: str = t("", args.repository_name, "KLOC")
            filename: str = appendID(filename=args.output, id="kloc_acceleration")
            title = title.format(
                "Acceleration of ",
                args.repository_name,
                "KLOC",
                args.stepper,
            )
            _graphAccelerationChart(
                title=title,
                yLabel=yLabel2.format("KLOC"),
                yData=yKLOC,
                filename=filename,
            )

        if args.graph_all:
            title: str = t("", args.repository_name, "KLOC")
            filename: str = appendID(filename=args.output, id="kloc_all")
            yLabelList: list = [
                yLabel0.format("KLOC"),
                yLabel0.format("KLOC"),
                yLabel1.format("KLOC"),
                yLabel2.format("KLOC"),
            ]
            _graphAllCharts(
                title=title,
                yLabelList=yLabelList,
                yData=yKLOC,
                filename=filename,
            )


if __name__ == "__main__":
    main()



from setuptools import setup

from ssl_metrics_git_commits_loc import version

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="ssl-metrics-git-commits-loc",
    packages=["ssl_metrics_git_commits_loc"],
    version=version.version(),
    description="SSL Metrics - Git History (LOC/KLOC) Analysis",
    author="Software and Systems Laboratory - Loyola University Chicago",
    author_email="ssl-metrics@ssl.luc.edu",
    license="BSD",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://ssl.cs.luc.edu/projects/metricsDashboard",
    project_urls={
        "Bug Tracker": "https://github.com/SoftwareSystemsLaboratory/ssl-metrics-git-commits-loc/issues",
        "GitHub Repository": "https://github.com/SoftwareSystemsLaboratory/ssl-metrics-git-commits-loc",
    },
    keywords=[
        "bus factor",
        "commits",
        "engineering",
        "git",
        "github",
        "issue density",
        "issues",
        "kloc",
        "loyola",
        "loyola university chicago",
        "luc",
        "mining",
        "metrics",
        "repository",
        "repository mining",
        "simple",
        "software",
        "software engineering",
        "software metrics",
        "software systems laboratory",
        "ssl"
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Software Development",
        "Topic :: Scientific/Engineering",
        "Operating System :: POSIX",
        "Operating System :: Unix",
        "Operating System :: MacOS",
    ],
    python_requires=">=3.9",
    install_requires=[
        "matplotlib>=3.4.3",
        "numpy>=1.21.2",
        "pandas>=1.3.3",
        "progress>=1.6",
        "python-dateutil>=2.8.2",
        "scikit-learn>=1.0.1",
    ],
    entry_points={
        "console_scripts": [
            "ssl-metrics-git-commits-loc-extract = ssl_metrics_git_commits_loc.main:main",
            "ssl-metrics-git-commits-loc-graph = ssl_metrics_git_commits_loc.create_graph:main",
        ]
    },
)



from operator import itemgetter

import numpy as np
from sklearn.metrics import r2_score


def findBestFitLine(x: list, y: list, maximumDegree: int) -> tuple:
    # https://www.w3schools.com/Python/python_ml_polynomial_regression.asp
    data: list = []

    degree: int
    for degree in range(maximumDegree):
        model: np.poly1d = np.poly1d(np.polyfit(x, y, degree))
        r2Score: np.float64 = r2_score(y, model(x))
        temp: tuple = (r2Score, model)
        data.append(temp)

    return max(data, key=itemgetter(0))
